/* 
 * Copyright (C) 2005 Remco Poelstra <remco@beryllium.net>
 *
 * LinABX - callbacks.c
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Authors: Remco Poelstra <remco@beryllium.net>
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>
#include <math.h>
#include "callbacks.h"
#include "interface.h"
#include "support.h"
#include "jack.h"
#include "sample_mode.h"
#include <jack/jack.h>
#include <sndfile.h>
#include <gconf/gconf-client.h>
//#include <glib-object.h>

int trials=0,correct=0,X=A,answer=NO_ANSWER;
extern enum mode_t mode;
extern GConfClient *gconf_client;
extern int restart;
extern int repeating;

GtkWidget *about=NULL;
GtkWidget *configure_dialog=NULL;
GtkWidget *finished_window=NULL;
GtkWidget *preferences=NULL;

char **ports_from_options=NULL;

char *A_filename=NULL;
char *B_filename=NULL;

double fac(int x) {
    int i;
    double result = 1;
    for(i=1; i<= x; i++)
	result = result * i;
    return result;
}

double ncr(int x, int y) {
    return (fac(x)/(fac(y)*fac((x-y))));
}

extern char *sample_mode_failures[];

int converter_type=-1;

int pop=0;
guint default_folder_id;
guint default_folder_setting_id;
guint default_mode_id;
guint restart_play_id;
guint repeating_id;
guint statistics_id;
guint default_set_id;
guint only_reference_id;
guint a_sample_id;
guint b_sample_id;
guint audibility_threshold_id;

void filter_add_sndfile_files(GtkFileFilter *filter) {
    gtk_file_filter_add_mime_type(filter,"audio/x-wav");
    gtk_file_filter_add_mime_type(filter,"audio/x-aiff");
    gtk_file_filter_add_mime_type(filter,"audio/basic");
    gtk_file_filter_add_pattern(filter,"*.raw");
    gtk_file_filter_add_pattern(filter,"*.paf");
    gtk_file_filter_add_pattern(filter,"*.iff");
    gtk_file_filter_add_pattern(filter,"*.svx");
    gtk_file_filter_add_pattern(filter,"*.sf");
    gtk_file_filter_add_pattern(filter,"*.voc");
    gtk_file_filter_add_pattern(filter,"*.w64");
    gtk_file_filter_add_pattern(filter,"*.mat4");
    gtk_file_filter_add_pattern(filter,"*.mat5");
    gtk_file_filter_add_pattern(filter,"*.pvf");
    gtk_file_filter_add_pattern(filter,"*.xi");
    gtk_file_filter_add_pattern(filter,"*.htk");
}

void
on_about_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    if(about) {
	gtk_window_present(GTK_WINDOW(about));
    } else {
	about=create_about();
	gtk_widget_show(about);
    }
}

void
on_about_close                         (GtkDialog       *dialog,
                                        gint             response_id,
                                        gpointer         user_data)
{
//    gtk_widget_destroy(GTK_WIDGET(dialog));
    about=NULL;
}

gboolean
on_mainwindow_quit                     (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
    gtk_main_quit();
  return FALSE;
}


void
on_a_clicked                           (GtkButton       *button,
                                        gpointer         user_data)
{
    char *text;
    GtkStatusbar *statusbar=GTK_STATUSBAR(lookup_widget(GTK_WIDGET(button),"statusbar"));
    
    answer=A;
    gtk_widget_set_sensitive(lookup_widget(GTK_WIDGET(button),"next"),TRUE);
    gtk_widget_set_sensitive(lookup_widget(GTK_WIDGET(button),"finish_session"),TRUE);
    if(restart)
	restart_play();
    set_playing(A);
    if(pop) gtk_statusbar_pop(statusbar,gtk_statusbar_get_context_id(statusbar,"answer"));
    text=g_strdup_printf("Trial: %d, Your current answer is: A",trials+1);
    gtk_statusbar_push(statusbar,gtk_statusbar_get_context_id(statusbar,"answer"),text);
    pop=1;
    g_free(text);
}


void
on_x_clicked                           (GtkButton       *button,
                                        gpointer         user_data)
{
    if(restart)
	restart_play();
    set_playing(X);
}


void
on_b_clicked                           (GtkButton       *button,
                                        gpointer         user_data)
{
    char *text;
    GtkStatusbar *statusbar=GTK_STATUSBAR(lookup_widget(GTK_WIDGET(button),"statusbar"));
    
    answer=B;
    gtk_widget_set_sensitive(lookup_widget(GTK_WIDGET(button),"next"),TRUE);
    gtk_widget_set_sensitive(lookup_widget(GTK_WIDGET(button),"finish_session"),TRUE);
    if(restart)
	restart_play();
    set_playing(B);
    if(pop) gtk_statusbar_pop(statusbar,gtk_statusbar_get_context_id(statusbar,"answer"));
    text=g_strdup_printf("Trial: %d, Your current answer is: B",trials+1);
    gtk_statusbar_push(statusbar,gtk_statusbar_get_context_id(statusbar,"answer"),text);    
    g_free(text);
    pop=1;
}


void
on_next_clicked                        (GtkButton       *button,
                                        gpointer         user_data)
{
    char *text;
    GtkStatusbar *statusbar=GTK_STATUSBAR(lookup_widget(GTK_WIDGET(button),"statusbar"));
        
    trials++;
    correct+=(answer==X?1:0);
    answer=NO_ANSWER;
    gtk_widget_set_sensitive(lookup_widget(GTK_WIDGET(button),"next"),FALSE);

    if(pop) gtk_statusbar_pop(statusbar,gtk_statusbar_get_context_id(statusbar,"answer"));
    pop=0;
    gtk_statusbar_pop(statusbar,gtk_statusbar_get_context_id(statusbar,"trials"));
    text=g_strdup_printf("Trial: %d",trials+1);
    gtk_statusbar_push(statusbar,gtk_statusbar_get_context_id(statusbar,"trials"),text);    
    g_free(text);
    set_playing(NO_ANSWER);
    X=(int) ((double)rand()/(double)(RAND_MAX+1.0)*2.0);
}


void
on_restart_clicked                     (GtkButton       *button,
                                        gpointer         user_data)
{
    GtkStatusbar *statusbar=GTK_STATUSBAR(lookup_widget(GTK_WIDGET(button),"statusbar"));

    trials=0;
    correct=0;
    answer=NO_ANSWER;
    set_playing(NO_ANSWER);
    X=(int) ((double)rand()/(double)(RAND_MAX+1.0)*2.0);
    gtk_statusbar_pop(statusbar,gtk_statusbar_get_context_id(statusbar,"trials"));
    gtk_statusbar_push(statusbar,gtk_statusbar_get_context_id(statusbar,"trials"),"Trial: 1");    
    gtk_label_set_text_with_mnemonic(GTK_LABEL(lookup_widget(GTK_WIDGET(button),"restart_session_label")),"_Restart Session");    
    gtk_widget_set_sensitive(lookup_widget(GTK_WIDGET(button),"next"),FALSE);
    gtk_widget_set_sensitive(lookup_widget(GTK_WIDGET(button),"finish_session"),FALSE);
    gtk_widget_set_sensitive(lookup_widget(GTK_WIDGET(button),"a"),TRUE);
    gtk_widget_set_sensitive(lookup_widget(GTK_WIDGET(button),"x"),TRUE);
    gtk_widget_set_sensitive(lookup_widget(GTK_WIDGET(button),"b"),TRUE);
}

void
on_finish_session_clicked              (GtkButton       *button,
                                        gpointer         user_data)
{
    GtkStatusbar *statusbar=GTK_STATUSBAR(lookup_widget(GTK_WIDGET(button),"statusbar"));
    GConfValue *value=gconf_client_get(gconf_client,"/apps/LinABX/general/statistics_use_normal",NULL);

    float binom=0, threshold;
    char *text;
    int i;

    set_playing(NO_ANSWER);
    /*If the user provided an answer then count it*/
    if(answer!=NO_ANSWER) {
	trials++;
	correct+=(answer==X?1:0);
    }
    
    /*Do the calculation*/
    if(!value||!(value->type==GCONF_VALUE_BOOL)||!gconf_value_get_bool(value)) {
	for ( i=0; i<=correct; i++) 
    	    binom = binom + ncr(trials,i)*pow(.5,i)*pow(.5,(trials-i));
    } else {
	float max=ncr(trials,trials/2)*pow(.5,trials/2)*pow(.5,trials/2);
	binom=1-(ncr(trials,correct)*pow(.5,correct)*pow(.5,(trials-correct)))/max;
	printf("trials:%d, correct:%d, half:%d, max:%f, binom:%f\n",trials,correct,trials/2,max,binom);
    }

    value=gconf_client_get(gconf_client,"/apps/LinABX/general/audibility_threshold",NULL);
    if(!value||value->type!=GCONF_VALUE_FLOAT) threshold=0.05;
    else threshold=gconf_value_get_float(value);
    
    /*Display the result*/
    if(finished_window==NULL)
	finished_window=create_finished_dialog();

    text=g_strdup_printf("<span weight=\"bold\" size=\"larger\">%d%% correct (p=%.3f)</span>\n\n\
You had <span weight=\"bold\">%d</span> answer%s of <span %s weight=\"bold\">%d</span> trial%s correct, which gives a correctness of <span weight=\"bold\">%d%%</span>.\n\
Compared to a coin-flip experiment, there is a chance of <span weight=\"bold\">%.3f</span> that this happens. Given the fact that this is %s than\
 <span weight=\"bold\">%.3f</span>, this can be seen as a %slikely situation.\n%s\n%s",
 (int) ((((float) correct)/((float) trials))*100),1-binom,correct,correct==1?"":"s",trials<16?"color=\"red\"":"",trials,trials==1?"":"s",(int) ((((float) correct)/((float) trials))*100),1-binom,
 1-binom<threshold?"smaller":"bigger",threshold,1-binom<threshold?"un":"",
 trials<16?"<span style=\"italic\">Given the fact that the number of trials is smaller than 16, these results may be highly inaccurate.</span>\n":"",
 1-binom<threshold?"<span color=\"blue\">You are <span style=\"italic\">probably</span> able to hear the difference.</span>":"<span color=\"red\">You are <span style=\"italic\">probably</span> unable to hear the difference.</span>");
//    text=g_strdup_printf("%d%% correctness (p = %.3f)",(int) ((((float) correct)/((float) trials))*100),1-binom);
    gtk_label_set_markup(GTK_LABEL(lookup_widget(GTK_WIDGET(finished_window),"probability")),text);    
    g_free(text);
    gtk_widget_show(finished_window);

    /*Reset the session for a new one*/
    trials=0;
    correct=0;
    answer=NO_ANSWER;
    X=(int) ((double)rand()/(double)(RAND_MAX+1.0)*2.0);
    gtk_statusbar_pop(statusbar,gtk_statusbar_get_context_id(statusbar,"trials"));
    gtk_statusbar_pop(statusbar,gtk_statusbar_get_context_id(statusbar,"answer"));
    gtk_label_set_text_with_mnemonic(GTK_LABEL(lookup_widget(GTK_WIDGET(button),"restart_session_label")),"_Start Session");    
    gtk_widget_set_sensitive(lookup_widget(GTK_WIDGET(button),"next"),FALSE);
    gtk_widget_set_sensitive(lookup_widget(GTK_WIDGET(button),"finish_session"),FALSE);
    gtk_widget_set_sensitive(lookup_widget(GTK_WIDGET(button),"a"),FALSE);
    gtk_widget_set_sensitive(lookup_widget(GTK_WIDGET(button),"x"),FALSE);
    gtk_widget_set_sensitive(lookup_widget(GTK_WIDGET(button),"b"),FALSE);
}


void
on_finished_dialog_response            (GtkDialog       *dialog,
                                        gint             response_id,
                                        gpointer         user_data)
{
    gtk_widget_destroy(finished_window);
    finished_window=NULL;
}

void
on_tone_activate                       (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    if(!gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(menuitem))) return;

    if(configure_dialog) {
	gtk_widget_destroy(configure_dialog);
	configure_dialog=NULL;
    }
}


void
on_jack_activate                       (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    GtkWidget *combo_box_al,*combo_box_ar,*combo_box_bl,*combo_box_br;
    GtkCheckMenuItem *item;
    const char **ports;
    int i,al,ar,bl,br;

    if(!gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(menuitem))) return;

    if(configure_dialog) {
	if(mode!=JACK_MODE) {
	    gtk_widget_destroy(configure_dialog);
	    configure_dialog=NULL;
	} else gtk_window_present(GTK_WINDOW(configure_dialog));
    }

    switch(mode) {
	case SAMPLE_MODE:
	    cleanup_sample_mode();
	    if(init_jack_mode()) {
		gtk_widget_set_sensitive(lookup_widget(GTK_WIDGET(menuitem),"restart"),TRUE);
		gtk_widget_set_sensitive(lookup_widget(GTK_WIDGET(menuitem),"properties"),TRUE);
		item=GTK_CHECK_MENU_ITEM(lookup_widget(GTK_WIDGET(menuitem),"sample"));
    		gtk_check_menu_item_set_inconsistent(item,FALSE);
		item=GTK_CHECK_MENU_ITEM(lookup_widget(GTK_WIDGET(menuitem),"jack"));
		gtk_check_menu_item_set_inconsistent(item,FALSE);
//    	    	item=GTK_CHECK_MENU_ITEM(lookup_widget(GTK_WIDGET(menuitem),"tone"));
//	    	gtk_check_menu_item_set_inconsistent(item,FALSE);
	    }
	    break;
	case TONE_MODE:
	    if(init_jack_mode()) {
		gtk_widget_set_sensitive(lookup_widget(GTK_WIDGET(menuitem),"restart"),TRUE);
		gtk_widget_set_sensitive(lookup_widget(GTK_WIDGET(menuitem),"properties"),TRUE);
		item=GTK_CHECK_MENU_ITEM(lookup_widget(GTK_WIDGET(menuitem),"sample"));
    		gtk_check_menu_item_set_inconsistent(item,FALSE);
		item=GTK_CHECK_MENU_ITEM(lookup_widget(GTK_WIDGET(menuitem),"jack"));
		gtk_check_menu_item_set_inconsistent(item,FALSE);
//    	    	item=GTK_CHECK_MENU_ITEM(lookup_widget(GTK_WIDGET(menuitem),"tone"));
//	    	gtk_check_menu_item_set_inconsistent(item,FALSE);
	    }
	    break;
	case JACK_MODE:
	    break;
	case NO_MODE:
	    if(init_jack_mode()) {
		gtk_widget_set_sensitive(lookup_widget(GTK_WIDGET(menuitem),"restart"),TRUE);
		gtk_widget_set_sensitive(lookup_widget(GTK_WIDGET(menuitem),"properties"),TRUE);
		item=GTK_CHECK_MENU_ITEM(lookup_widget(GTK_WIDGET(menuitem),"sample"));
    		gtk_check_menu_item_set_inconsistent(item,FALSE);
		item=GTK_CHECK_MENU_ITEM(lookup_widget(GTK_WIDGET(menuitem),"jack"));
		gtk_check_menu_item_set_inconsistent(item,FALSE);
//    	    	item=GTK_CHECK_MENU_ITEM(lookup_widget(GTK_WIDGET(menuitem),"tone"));
//	    	gtk_check_menu_item_set_inconsistent(item,FALSE);
	    }
	    break;
    }
    if(configure_dialog==NULL) {
        configure_dialog=create_configure_jack_mode_dialog();
        combo_box_al=lookup_widget(GTK_WIDGET(configure_dialog),"A_left_combo");
        combo_box_ar=lookup_widget(GTK_WIDGET(configure_dialog),"A_right_combo");
        combo_box_bl=lookup_widget(GTK_WIDGET(configure_dialog),"B_left_combo");
        combo_box_br=lookup_widget(GTK_WIDGET(configure_dialog),"B_right_combo");
        ports=get_available_ports(&al,&ar,&bl,&br);
        for(i=0;ports[i];i++) {
            if(!strncmp(ports[i],"linabx:",7)) continue;
            gtk_combo_box_append_text(GTK_COMBO_BOX(combo_box_al),ports[i]);
            gtk_combo_box_append_text(GTK_COMBO_BOX(combo_box_ar),ports[i]);
            gtk_combo_box_append_text(GTK_COMBO_BOX(combo_box_bl),ports[i]);
            gtk_combo_box_append_text(GTK_COMBO_BOX(combo_box_br),ports[i]);
        }
        gtk_combo_box_set_active(GTK_COMBO_BOX(combo_box_al),al);
        gtk_combo_box_set_active(GTK_COMBO_BOX(combo_box_ar),ar);
        gtk_combo_box_set_active(GTK_COMBO_BOX(combo_box_bl),bl);
        gtk_combo_box_set_active(GTK_COMBO_BOX(combo_box_br),br);
	if(ports_from_options) {
	    for(i=0;ports[i]&&ports_from_options[0];i++) {
		if(strcmp(ports_from_options[0],ports[i])==0)
		    gtk_combo_box_set_active(GTK_COMBO_BOX(combo_box_al),i);
	    }
	    for(i=0;ports[i]&&ports_from_options[1];i++) {
		if(strcmp(ports_from_options[1],ports[i])==0)
		    gtk_combo_box_set_active(GTK_COMBO_BOX(combo_box_ar),i);
	    }
	    for(i=0;ports[i]&&ports_from_options[2];i++) {
		if(strcmp(ports_from_options[2],ports[i])==0)
		    gtk_combo_box_set_active(GTK_COMBO_BOX(combo_box_bl),i);
	    }
	    for(i=0;ports[i]&&ports_from_options[3];i++) {
		if(strcmp(ports_from_options[3],ports[i])==0)
		    gtk_combo_box_set_active(GTK_COMBO_BOX(combo_box_br),i);
	    }
	    for(i=0;i<4;i++)
		g_free(ports_from_options[i]);
	    g_free(ports_from_options);
	    ports_from_options=NULL;
	}
	gtk_widget_show(configure_dialog);
    }
}

void
on_properties_activate                 (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    if(configure_dialog) {
	gtk_window_present(GTK_WINDOW(configure_dialog));
    } else {
	switch(mode) {
	    case SAMPLE_MODE:
		gtk_menu_item_activate(GTK_MENU_ITEM(lookup_widget(GTK_WIDGET(menuitem),"sample")));
		break;
	    case JACK_MODE:    
		gtk_menu_item_activate(GTK_MENU_ITEM(lookup_widget(GTK_WIDGET(menuitem),"jack")));
		break;
	    case TONE_MODE:
		gtk_menu_item_activate(GTK_MENU_ITEM(lookup_widget(GTK_WIDGET(menuitem),"tone")));
		break;
	    case NO_MODE:
		break;
	}
    }
}

void
on_configure_jack_mode_dialog_response (GtkDialog       *dialog,
                                        gint             response_id,
                                        gpointer         user_data)
{
    if(response_id!=GTK_RESPONSE_HELP) {
        gtk_widget_destroy(configure_dialog);
	configure_dialog=NULL;
    }
}


void
on_A_left_combo_changed                (GtkComboBox     *combobox,
                                        gpointer         user_data)
{
    char *port_name=gtk_combo_box_get_active_text(GTK_COMBO_BOX(combobox));
    
    if(port_name) {
        connect_port(0,port_name);
    }

}


void
on_A_right_combo_changed               (GtkComboBox     *combobox,
                                        gpointer         user_data)
{
    char *port_name=gtk_combo_box_get_active_text(GTK_COMBO_BOX(combobox));

    if(port_name) {
        connect_port(1,port_name);
    }
}


void
on_B_left_combo_changed                (GtkComboBox     *combobox,
                                        gpointer         user_data)
{
    char *port_name=gtk_combo_box_get_active_text(GTK_COMBO_BOX(combobox));
    
    if(port_name) {
        connect_port(2,port_name);
    }
}


void
on_B_right_combo_changed               (GtkComboBox     *combobox,
                                        gpointer         user_data)
{
    char *port_name=gtk_combo_box_get_active_text(GTK_COMBO_BOX(combobox));
    
    if(port_name) {
        connect_port(3,port_name);
    }
}

void
on_configure_sample_mode_dialog_response
                                        (GtkDialog       *dialog,
                                        gint             response_id,
                                        gpointer         user_data)
{
    GtkStatusbar *statusbar=GTK_STATUSBAR(lookup_widget(GTK_WIDGET(g_object_get_data(G_OBJECT(dialog),"mainwindow")),"statusbar"));
    GtkCheckMenuItem *item;
    int result;
    
    if(response_id==GTK_RESPONSE_OK) {
	switch(mode) {
	    case SAMPLE_MODE:	
		cleanup_sample_mode();
		break;
	    case TONE_MODE:
		break;
	    case JACK_MODE:
	        cleanup_jack_mode();
	        break;
	    case NO_MODE:
	        break;
        }
	if(!(converter_type>=0&&converter_type<=4)) {
	    GConfValue *value=gconf_client_get(gconf_client,"/apps/LinABX/samples/sample_rate_converter",NULL);
	    if(value&&(value->type==GCONF_VALUE_INT)) {
		int type=gconf_value_get_int(value);
		if(!(type>=0&&type<=4)) converter_type=1;
		else converter_type=type;
	    } else converter_type=1;
	}
	result=init_sample_mode(A_filename,B_filename,converter_type);

	if(result) {
	    gtk_widget_set_sensitive(lookup_widget(GTK_WIDGET(g_object_get_data(G_OBJECT(dialog),"mainwindow")),"restart"),TRUE);
	    gtk_widget_set_sensitive(lookup_widget(GTK_WIDGET(g_object_get_data(G_OBJECT(dialog),"mainwindow")),"properties"),TRUE);

            item=GTK_CHECK_MENU_ITEM(lookup_widget(g_object_get_data(G_OBJECT(dialog),"mainwindow"),"sample"));
    	    gtk_check_menu_item_set_inconsistent(item,FALSE);
	    item=GTK_CHECK_MENU_ITEM(lookup_widget(g_object_get_data(G_OBJECT(dialog),"mainwindow"),"jack"));
	    gtk_check_menu_item_set_inconsistent(item,FALSE);
//    	    item=GTK_CHECK_MENU_ITEM(lookup_widget(g_object_get_data(G_OBJECT(dialog),"mainwindow"),"tone"));
//	    gtk_check_menu_item_set_inconsistent(item,FALSE);
	    gtk_widget_destroy(configure_dialog);
	    configure_dialog=NULL;
	    if(result==2) 
		gtk_statusbar_push(statusbar,gtk_statusbar_get_context_id(statusbar,"jack"),"Warning: Samplerate of Jack and sample set differ, using samplerate converter");
	    else 
		gtk_statusbar_pop(statusbar,gtk_statusbar_get_context_id(statusbar,"jack"));
	} else {
	    gtk_label_set_markup(GTK_LABEL(lookup_widget(GTK_WIDGET(dialog),"valid_sample_set_label")),"<span weight=\"bold\" color=\"red\">Initializing sample mode failed</span>");
	}
    } else if (response_id==GTK_RESPONSE_HELP){
    
    } else {
        gtk_widget_destroy(configure_dialog);
	configure_dialog=NULL;
	/*We are not in sample mode, but the radio menuitem thinks we are. Correct for that */
	switch(mode) {
	    case JACK_MODE:
	        item=GTK_CHECK_MENU_ITEM(lookup_widget(g_object_get_data(G_OBJECT(dialog),"mainwindow"),"jack"));
		g_signal_handlers_block_by_func(G_OBJECT(item),on_jack_activate,NULL);
		gtk_check_menu_item_set_active(item,TRUE);
	        g_signal_handlers_unblock_by_func(G_OBJECT(item),on_jack_activate,NULL);
		break;
	    case TONE_MODE:
	        item=GTK_CHECK_MENU_ITEM(lookup_widget(g_object_get_data(G_OBJECT(dialog),"mainwindow"),"tone"));
		g_signal_handlers_block_by_func(G_OBJECT(item),on_tone_activate,NULL);
		gtk_check_menu_item_set_active(item,TRUE);
	        g_signal_handlers_unblock_by_func(G_OBJECT(item),on_tone_activate,NULL);
		break;
	    case NO_MODE:
	        item=GTK_CHECK_MENU_ITEM(lookup_widget(g_object_get_data(G_OBJECT(dialog),"mainwindow"),"sample"));
		gtk_check_menu_item_set_inconsistent(item,TRUE);
	        item=GTK_CHECK_MENU_ITEM(lookup_widget(g_object_get_data(G_OBJECT(dialog),"mainwindow"),"jack"));
		gtk_check_menu_item_set_inconsistent(item,TRUE);
//	        item=GTK_CHECK_MENU_ITEM(lookup_widget(g_object_get_data(G_OBJECT(dialog),"mainwindow"),"tone"));
//		gtk_check_menu_item_set_inconsistent(item,TRUE);
		break;
	    default:
		break;
	}
    }
}

void
on_choose_mode_dialog_response         (GtkDialog       *dialog,
                                        gint             response_id,
                                        gpointer         user_data)
{
    switch(response_id) {
	case GTK_RESPONSE_HELP:
	    break;
	case 0:
	    gtk_main_quit();
	    break;
	case 1:
	    gtk_menu_item_activate(GTK_MENU_ITEM(lookup_widget(GTK_WIDGET(g_object_get_data(G_OBJECT(dialog),"mainwindow")),"tone")));
	    gtk_widget_destroy(GTK_WIDGET(dialog));
	    break;
	case 2:
	    gtk_menu_item_activate(GTK_MENU_ITEM(lookup_widget(GTK_WIDGET(g_object_get_data(G_OBJECT(dialog),"mainwindow")),"jack")));
	    gtk_widget_destroy(GTK_WIDGET(dialog));
	    break;
	case 3:
	    gtk_menu_item_activate(GTK_MENU_ITEM(lookup_widget(GTK_WIDGET(g_object_get_data(G_OBJECT(dialog),"mainwindow")),"sample")));
	    gtk_widget_destroy(GTK_WIDGET(dialog));
	    break;
	default:
	    gtk_main_quit();
	    break;
    }
}


void
on_best_radiobutton_toggled            (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
    if(gtk_toggle_button_get_active(togglebutton))
	gconf_client_set_int(gconf_client,"/apps/LinABX/samples/sample_rate_converter",0,NULL);
}


void
on_medium_radiobutton_toggled          (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
    if(gtk_toggle_button_get_active(togglebutton))
	gconf_client_set_int(gconf_client,"/apps/LinABX/samples/sample_rate_converter",1,NULL);
}


void
on_low_radiobutton_toggled             (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
    if(gtk_toggle_button_get_active(togglebutton))
	gconf_client_set_int(gconf_client,"/apps/LinABX/samples/sample_rate_converter",2,NULL);
}


void
on_zero_radiobutton_toggled            (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
    if(gtk_toggle_button_get_active(togglebutton))
	gconf_client_set_int(gconf_client,"/apps/LinABX/samples/sample_rate_converter",3,NULL);
}


void
on_linear_radiobutton_toggled          (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
    if(gtk_toggle_button_get_active(togglebutton))
	gconf_client_set_int(gconf_client,"/apps/LinABX/samples/sample_rate_converter",4,NULL);
}

void
on_A_filechooser_current_folder_changed
                                        (GtkFileChooser  *filechooser,
                                        gpointer         user_data)
{
    char *folder=gtk_file_chooser_get_current_folder(filechooser);
    char *file=gtk_file_chooser_get_filename(filechooser);
    GConfEntry *entry=gconf_client_get_entry(gconf_client,"/apps/LinABX/general/default_folder_setting",NULL,TRUE,NULL);
	
    if(file) {
	g_free(A_filename);
	A_filename=file;
	if(entry&&entry->value&&entry->value->type==GCONF_VALUE_INT&&gconf_value_get_int(entry->value)==1) {
	    gconf_client_set_string(gconf_client,"/apps/LinABX/general/default_folder",folder,NULL);
	}
    }
    if(!B_filename) {
	gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(lookup_widget(GTK_WIDGET(filechooser),"B_filechooser")),folder);
    }
    
    if(A_filename&&B_filename) {
	SF_INFO info;
	SNDFILE *sf;
	int i=0, srate=0, channels=0;
	
	info.channels=30; /*just a value*/
	info.samplerate=2;
	
	if((sf=sf_open(A_filename,SFM_READ,&info))) {
	    i++;
	    srate=info.samplerate;
	    channels=info.channels;
	    sf_close(sf);
	}
	if((sf=sf_open(B_filename,SFM_READ,&info))) {
	    i++;
	    sf_close(sf);
	}
	if(i==2&&srate==info.samplerate&&channels==info.channels) {
	    gtk_widget_set_sensitive(lookup_widget(GTK_WIDGET(filechooser),"okbutton"),TRUE);
	    gtk_label_set_markup(GTK_LABEL(lookup_widget(GTK_WIDGET(filechooser),"valid_sample_set_label")),"Valid sample set");
	} else if(i!=2) {
	    gtk_widget_set_sensitive(lookup_widget(GTK_WIDGET(filechooser),"okbutton"),FALSE);
	    gtk_label_set_markup(GTK_LABEL(lookup_widget(GTK_WIDGET(filechooser),"valid_sample_set_label")),"<span color=\"red\">No valid sample set given: Couldn't open files</span>");
	} else if(srate!=info.samplerate) {
	    gtk_widget_set_sensitive(lookup_widget(GTK_WIDGET(filechooser),"okbutton"),FALSE);
	    gtk_label_set_markup(GTK_LABEL(lookup_widget(GTK_WIDGET(filechooser),"valid_sample_set_label")),"<span color=\"red\">No valid sample set given: Samplerates don't match</span>");
	} else if(channels!=info.channels) {
	    gtk_widget_set_sensitive(lookup_widget(GTK_WIDGET(filechooser),"okbutton"),FALSE);
	    gtk_label_set_markup(GTK_LABEL(lookup_widget(GTK_WIDGET(filechooser),"valid_sample_set_label")),"<span color=\"red\">No valid sample set given: Number of channels doesn't match</span>");
	}
    }
}


void
on_B_filechooser_current_folder_changed
                                        (GtkFileChooser  *filechooser,
                                        gpointer         user_data)
{
    char *folder=gtk_file_chooser_get_current_folder(filechooser);
    char *file=gtk_file_chooser_get_filename(filechooser);
    GConfEntry *entry=gconf_client_get_entry(gconf_client,"/apps/LinABX/general/default_folder_setting",NULL,TRUE,NULL);
    
    if(file) {
	g_free(B_filename);
	B_filename=file;
	if(entry&&entry->value&&entry->value->type==GCONF_VALUE_INT&&gconf_value_get_int(entry->value)==1) {
	    gconf_client_set_string(gconf_client,"/apps/LinABX/general/default_folder",folder,NULL);
	}
    }
    if(!A_filename) {
	gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(lookup_widget(GTK_WIDGET(filechooser),"A_filechooser")),folder);
    }
    if(A_filename&&B_filename) {
	SF_INFO info;
	SNDFILE *sf;
	int i=0, srate=0, channels=0;

	info.channels=30; /*just a value*/
	info.samplerate=2;
	
	if((sf=sf_open(A_filename,SFM_READ,&info))) {
	    i++;
	    srate=info.samplerate;
	    channels=info.channels;
	    sf_close(sf);
	}
	if((sf=sf_open(B_filename,SFM_READ,&info))) {
	    i++;
	    sf_close(sf);
	}
	if(i==2&&srate==info.samplerate&&channels==info.channels) {
	    gtk_widget_set_sensitive(lookup_widget(GTK_WIDGET(filechooser),"okbutton"),TRUE);
	    gtk_label_set_markup(GTK_LABEL(lookup_widget(GTK_WIDGET(filechooser),"valid_sample_set_label")),"Valid sample set");
	} else if(i!=2) {
	    gtk_widget_set_sensitive(lookup_widget(GTK_WIDGET(filechooser),"okbutton"),FALSE);
	    gtk_label_set_markup(GTK_LABEL(lookup_widget(GTK_WIDGET(filechooser),"valid_sample_set_label")),"<span color=\"red\">No valid sample set given: Couldn't open files</span>");
	} else if(srate!=info.samplerate) {
	    gtk_widget_set_sensitive(lookup_widget(GTK_WIDGET(filechooser),"okbutton"),FALSE);
	    gtk_label_set_markup(GTK_LABEL(lookup_widget(GTK_WIDGET(filechooser),"valid_sample_set_label")),"<span color=\"red\">No valid sample set given: Samplerates don't match</span>");
	} else if(channels!=info.channels) {
	    gtk_widget_set_sensitive(lookup_widget(GTK_WIDGET(filechooser),"okbutton"),FALSE);
	    gtk_label_set_markup(GTK_LABEL(lookup_widget(GTK_WIDGET(filechooser),"valid_sample_set_label")),"<span color=\"red\">No valid sample set given: Number of channels doesn't match</span>");
	}
    }
}

void
on_sample_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    GtkFileFilter *filter;
    if(!gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(menuitem))) return;
    if(configure_dialog) {
	if(mode!=SAMPLE_MODE) {
	    gtk_widget_destroy(configure_dialog);
	    configure_dialog=NULL;
	} else gtk_window_present(GTK_WINDOW(configure_dialog));
    }
    if(configure_dialog==NULL) {
	GtkFileChooser *A_filechooser;
	GtkFileChooser *B_filechooser;
	
	/*If a default sample set is requested, we fill in the filenames here, but only if the filenames aren't set allready*/
	
	if(!A_filename) {
	    GConfValue *value=gconf_client_get(gconf_client,"/apps/LinABX/samples/default_set",NULL);
	    if(value&&value->type==GCONF_VALUE_INT&&gconf_value_get_int(value)==1) {
		value=gconf_client_get(gconf_client,"/apps/LinABX/samples/default_a_sample",NULL);
		if(value&&value->type==GCONF_VALUE_STRING)
		    A_filename=g_strdup(gconf_value_get_string(value));
	    }
	}
	if(!B_filename) {
	    GConfValue *value=gconf_client_get(gconf_client,"/apps/LinABX/samples/default_set",NULL);
	    if(value&&value->type==GCONF_VALUE_INT&&gconf_value_get_int(value)==1) {
		value=gconf_client_get(gconf_client,"/apps/LinABX/samples/only_reference_sample",NULL);
		if(value&&value->type==GCONF_VALUE_BOOL&&!gconf_value_get_bool(value)) {
		    value=gconf_client_get(gconf_client,"/apps/LinABX/samples/default_b_sample",NULL);
		    if(value&&value->type==GCONF_VALUE_STRING)
			B_filename=g_strdup(gconf_value_get_string(value));
		}
	    }
	}
	configure_dialog=create_configure_sample_mode_dialog();
	g_object_set_data(G_OBJECT(configure_dialog),"mainwindow",lookup_widget(GTK_WIDGET(menuitem),"mainwindow"));
	A_filechooser=GTK_FILE_CHOOSER(lookup_widget(GTK_WIDGET(configure_dialog),"A_filechooser"));
	B_filechooser=GTK_FILE_CHOOSER(lookup_widget(GTK_WIDGET(configure_dialog),"B_filechooser"));
	if(A_filename) gtk_file_chooser_set_filename(A_filechooser,A_filename);
	else {
	    GConfEntry *entry=gconf_client_get_entry(gconf_client,"/apps/LinABX/general/default_folder_setting",NULL,TRUE,NULL);
	    if(entry&&entry->value&&entry->value->type==GCONF_VALUE_INT&&gconf_value_get_int(entry->value)!=0) {
		GConfEntry *entry=gconf_client_get_entry(gconf_client,"/apps/LinABX/general/default_folder",NULL,TRUE,NULL);
		if(entry->value&&entry->value->type==GCONF_VALUE_STRING) {
		    g_signal_handlers_block_by_func(A_filechooser,on_A_filechooser_current_folder_changed,NULL);
		    gtk_file_chooser_set_current_folder(A_filechooser,gconf_value_get_string(entry->value));
		    g_signal_handlers_unblock_by_func(A_filechooser,on_A_filechooser_current_folder_changed,NULL);
		}
	    }
	}
	if(B_filename) gtk_file_chooser_set_filename(B_filechooser,B_filename);
	else {
	    GConfEntry *entry=gconf_client_get_entry(gconf_client,"/apps/LinABX/general/default_folder_setting",NULL,TRUE,NULL);
	    if(entry&&entry->value&&entry->value->type==GCONF_VALUE_INT&&gconf_value_get_int(entry->value)!=0) {
		GConfEntry *entry=gconf_client_get_entry(gconf_client,"/apps/LinABX/general/default_folder",NULL,TRUE,NULL);
		if(entry&&entry->value&&entry->value->type==GCONF_VALUE_STRING) {
		    g_signal_handlers_block_by_func(B_filechooser,on_B_filechooser_current_folder_changed,NULL);
		    gtk_file_chooser_set_current_folder(B_filechooser,gconf_value_get_string(entry->value));
		    g_signal_handlers_unblock_by_func(B_filechooser,on_B_filechooser_current_folder_changed,NULL);
		}
	    }
	}
	filter=gtk_file_filter_new();
	filter_add_sndfile_files(filter);
	gtk_file_filter_set_name(filter,"Audio Files");
	gtk_file_chooser_add_filter(A_filechooser,filter);
	filter=gtk_file_filter_new();
	gtk_file_filter_add_pattern(filter,"*");
	gtk_file_filter_set_name(filter,"All Files");
	gtk_file_chooser_add_filter(A_filechooser,filter);
	filter=gtk_file_filter_new();
	filter_add_sndfile_files(filter);
	gtk_file_filter_set_name(filter,"Audio Files");
	gtk_file_chooser_add_filter(B_filechooser,filter);
	filter=gtk_file_filter_new();
	gtk_file_filter_add_pattern(filter,"*");
	gtk_file_filter_set_name(filter,"All Files");
	gtk_file_chooser_add_filter(B_filechooser,filter);
	gtk_widget_show(configure_dialog);
    }
}

void default_folder_setting_changed(GConfClient *client, guint id, GConfEntry *entry, gpointer user_data) {
    if(!entry||!entry->value||entry->value->type!=GCONF_VALUE_INT) {
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(lookup_widget(GTK_WIDGET(preferences),"folder_not_change_radiobutton")),TRUE);
    } else {
	switch(gconf_value_get_int(entry->value)) {
	    case 0:
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(lookup_widget(GTK_WIDGET(preferences),"folder_not_change_radiobutton")),TRUE);
		break;
	    case 1:
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(lookup_widget(GTK_WIDGET(preferences),"folder_goto_last_radiobutton")),TRUE);
		break;
	    case 2:
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(lookup_widget(GTK_WIDGET(preferences),"folder_set_to_radiobutton")),TRUE);
		break;
	    default:
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(lookup_widget(GTK_WIDGET(preferences),"folder_not_change_radiobutton")),TRUE);
		break;
	}
    }
}

void default_folder_changed(GConfClient *client, guint id, GConfEntry *entry, gpointer user_data) {
    if(!entry||entry->value==NULL||entry->value->type!=GCONF_VALUE_STRING) {
	gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(lookup_widget(GTK_WIDGET(preferences),"folder_set_to_filechooser")),g_get_home_dir());
    } else {
	GConfEntry *setting=gconf_client_get_entry(gconf_client,"/apps/LinABX/general/default_foler_setting",NULL,TRUE,NULL);
	if(!setting||!setting->value||setting->value->type!=GCONF_VALUE_INT||gconf_value_get_int(setting->value)==0) return;
	gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(lookup_widget(GTK_WIDGET(preferences),"folder_set_to_filechooser")),gconf_value_get_string(entry->value));
    }
}

void default_mode_changed(GConfClient *client, guint id, GConfEntry *entry, gpointer user_data) {
    if(!entry||!entry->value||entry->value->type!=GCONF_VALUE_INT) {
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(lookup_widget(GTK_WIDGET(preferences),"default_no_mode_radiobutton")),TRUE);
    } else {
	int mode=gconf_value_get_int(entry->value);
	switch(mode) {
	    case 0:
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(lookup_widget(GTK_WIDGET(preferences),"default_no_mode_radiobutton")),TRUE);
		break;
	    case 1:
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(lookup_widget(GTK_WIDGET(preferences),"default_sample_radiobutton")),TRUE);
		break;
	    case 2:
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(lookup_widget(GTK_WIDGET(preferences),"default_jack_radiobutton")),TRUE);
		break;
	    case 3:
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(lookup_widget(GTK_WIDGET(preferences),"default_tone_radiobutton")),TRUE);
		break;
	    default:
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(lookup_widget(GTK_WIDGET(preferences),"default_no_mode_radiobutton")),TRUE);
		break;
	}
    }
}

void restart_play_changed(GConfClient *client, guint id, GConfEntry *entry, gpointer user_data) {
    if(!entry||!entry->value||entry->value->type!=GCONF_VALUE_BOOL) {
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(lookup_widget(GTK_WIDGET(preferences),"restart_sample_checkbutton")),TRUE);
	restart=1; /*It's actually already set in sample_mode.c but better safe than sorry*/
    }
    else {
	if((restart=gconf_value_get_bool(entry->value))) /*directly assign the value used in sample_mode.c -> get_new_data() */
	    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(lookup_widget(GTK_WIDGET(preferences),"restart_sample_checkbutton")),TRUE);
	else
	    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(lookup_widget(GTK_WIDGET(preferences),"restart_sample_checkbutton")),FALSE);
    }
}

void repeating_changed(GConfClient *client, guint id, GConfEntry *entry, gpointer user_data) {
	if(!entry||!entry->value||entry->value->type!=GCONF_VALUE_BOOL) {
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(lookup_widget(GTK_WIDGET(preferences),"repeating_checkbutton")),TRUE);
		repeating=1;
	} else {
		if( (repeating=gconf_value_get_bool(entry->value)) )
	    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(lookup_widget(GTK_WIDGET(preferences),"repeating_checkbutton")),TRUE);
		else
	    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(lookup_widget(GTK_WIDGET(preferences),"repeating_checkbutton")),FALSE);
    }
}

void statistics_changed(GConfClient *client, guint id, GConfEntry *entry, gpointer user_data) {
    if(!entry||!entry->value||entry->value->type!=GCONF_VALUE_BOOL)
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(lookup_widget(GTK_WIDGET(preferences),"cumulative_radiobutton")),TRUE);
    else {
	if (gconf_value_get_bool(entry->value))
	    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(lookup_widget(GTK_WIDGET(preferences),"normal_radiobutton")),TRUE);
	else
	    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(lookup_widget(GTK_WIDGET(preferences),"cumulative_radiobutton")),TRUE);
    }
	
}

void converter_type_changed(GConfClient *client, guint id, GConfEntry *entry, gpointer user_data) {
    GConfValue *value=gconf_entry_get_value(entry);
    if(!value||!(value->type==GCONF_VALUE_INT)) {
	return;
    }
    int type=gconf_value_get_int(value);
    if((type>=0&&type<=4))
	converter_type=type;
    else converter_type=1;
    
    if(preferences) {
        switch(converter_type) {
	    case 0:
	        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(lookup_widget(preferences,"best_radiobutton")),TRUE);
	        break;
	    case 1:
	        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(lookup_widget(preferences,"medium_radiobutton")),TRUE);
	        break;
	    case 2:
	        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(lookup_widget(preferences,"low_radiobutton")),TRUE);
	        break;
	    case 3:
	        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(lookup_widget(preferences,"zero_radiobutton")),TRUE);
	        break;
	    case 4:
	        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(lookup_widget(preferences,"linear_radiobutton")),TRUE);
	        break;
	}
    }
    if(mode==SAMPLE_MODE) {
        cleanup_sample_mode();
        init_sample_mode(A_filename,B_filename,converter_type);
    }
}

void only_reference_changed(GConfClient *client, guint id, GConfEntry *entry, gpointer user_data) {
    if(!entry||!entry->value||entry->value->type!=GCONF_VALUE_BOOL||gconf_value_get_bool(entry->value)==0) {
	GConfValue *value=gconf_client_get(gconf_client,"/apps/LinABX/samples/default_set",NULL);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(lookup_widget(preferences,"only_reference_checkbutton")),FALSE);
	if(value&&value->type==GCONF_VALUE_INT&&gconf_value_get_int(value)==1) {
	    gtk_widget_set_sensitive(GTK_WIDGET(lookup_widget(preferences,"default_set_b_sample_label")),TRUE);
	    gtk_widget_set_sensitive(GTK_WIDGET(lookup_widget(preferences,"default_set_b_sample_filechooserbutton")),TRUE);
	}
    } else {
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(lookup_widget(preferences,"only_reference_checkbutton")),TRUE);
	gtk_widget_set_sensitive(GTK_WIDGET(lookup_widget(preferences,"default_set_b_sample_label")),FALSE);
	gtk_widget_set_sensitive(GTK_WIDGET(lookup_widget(preferences,"default_set_b_sample_filechooserbutton")),FALSE);
    }
}

void default_set_changed(GConfClient *client, guint id, GConfEntry *entry, gpointer user_data) {
    if(!entry||!entry->value||entry->value->type!=GCONF_VALUE_INT||gconf_value_get_int(entry->value)==0) {
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(lookup_widget(preferences,"default_set_none_radiobutton")),TRUE);
        gtk_widget_set_sensitive(GTK_WIDGET(lookup_widget(preferences,"only_reference_checkbutton")),FALSE);
	gtk_widget_set_sensitive(GTK_WIDGET(lookup_widget(preferences,"default_set_a_sample_label")),FALSE);
	gtk_widget_set_sensitive(GTK_WIDGET(lookup_widget(preferences,"default_set_b_sample_label")),FALSE);
	gtk_widget_set_sensitive(GTK_WIDGET(lookup_widget(preferences,"default_set_a_sample_filechooserbutton")),FALSE);
	gtk_widget_set_sensitive(GTK_WIDGET(lookup_widget(preferences,"default_set_b_sample_filechooserbutton")),FALSE);
    } else {
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(lookup_widget(preferences,"default_set_fixed_radiobutton")),TRUE);
	gtk_widget_set_sensitive(GTK_WIDGET(lookup_widget(preferences,"only_reference_checkbutton")),TRUE);
        gtk_widget_set_sensitive(GTK_WIDGET(lookup_widget(preferences,"default_set_a_sample_label")),TRUE);
        gtk_widget_set_sensitive(GTK_WIDGET(lookup_widget(preferences,"default_set_a_sample_filechooserbutton")),TRUE);
        only_reference_changed(gconf_client,-1,gconf_client_get_entry(gconf_client,"/apps/LinABX/samples/only_reference_sample",NULL,TRUE,NULL),NULL);
    }
}

void a_sample_changed(GConfClient *client, guint id, GConfEntry *entry, gpointer user_data) {
    if(!entry||!entry->value||entry->value->type!=GCONF_VALUE_STRING) return;
    gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(lookup_widget(preferences,"default_set_a_sample_filechooserbutton")),gconf_value_get_string(entry->value));
}

void b_sample_changed(GConfClient *client, guint id, GConfEntry *entry, gpointer user_data) {
    if(!entry||!entry->value||entry->value->type!=GCONF_VALUE_STRING) return;
    gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(lookup_widget(preferences,"default_set_b_sample_filechooserbutton")),gconf_value_get_string(entry->value));
}

void audibility_threshold_changed(GConfClient *client, guint id, GConfEntry *entry, gpointer user_data) {
    if(!entry||!entry->value||entry->value->type!=GCONF_VALUE_FLOAT)
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(lookup_widget(preferences,"audibility_threshold_spinbutton")),0.05);
    else
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(lookup_widget(preferences,"audibility_threshold_spinbutton")),gconf_value_get_float(entry->value));
}

void
on_preferences_activate                (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    if(preferences) {
	gtk_window_present(GTK_WINDOW(preferences));
    } else {
	preferences=create_preferences_dialog();
	default_folder_setting_id=gconf_client_notify_add(gconf_client,"/apps/LinABX/general/default_folder_setting",default_folder_setting_changed,NULL,NULL,NULL);
	default_folder_id=gconf_client_notify_add(gconf_client,"/apps/LinABX/general/default_folder",default_folder_changed,NULL,NULL,NULL);
	default_mode_id=gconf_client_notify_add(gconf_client,"/apps/LinABX/general/default_mode",default_mode_changed,NULL,NULL,NULL);
	restart_play_id=gconf_client_notify_add(gconf_client,"/apps/LinABX/samples/restart_play",restart_play_changed,NULL,NULL,NULL);
	repeating_id=gconf_client_notify_add(gconf_client,"/apps/LinABX/samples/repeating",repeating_changed,NULL,NULL,NULL);
	statistics_id=gconf_client_notify_add(gconf_client,"/apps/LinABX/general/statistics_use_normal",statistics_changed,NULL,NULL,NULL);
	default_set_id=gconf_client_notify_add(gconf_client,"/apps/LinABX/samples/default_set",default_set_changed,NULL,NULL,NULL);
	only_reference_id=gconf_client_notify_add(gconf_client,"/apps/LinABX/samples/only_reference_sample",only_reference_changed,NULL,NULL,NULL);
	a_sample_id=gconf_client_notify_add(gconf_client,"/apps/LinABX/samples/default_a_sample",a_sample_changed,NULL,NULL,NULL);
	b_sample_id=gconf_client_notify_add(gconf_client,"/apps/LinABX/samples/default_b_sample",b_sample_changed,NULL,NULL,NULL);
	audibility_threshold_id=gconf_client_notify_add(gconf_client,"/apps/LinABX/general/audibility_threshold",audibility_threshold_changed,NULL,NULL,NULL);

	default_folder_setting_changed(gconf_client,-1,gconf_client_get_entry(gconf_client,"/apps/LinABX/general/default_folder_setting",NULL,TRUE,NULL),NULL);
	default_folder_changed(gconf_client,-1,gconf_client_get_entry(gconf_client,"/apps/LinABX/general/default_folder",NULL,TRUE,NULL),NULL);
	converter_type_changed(gconf_client,-1,gconf_client_get_entry(gconf_client,"/apps/LinABX/general/sample_rate_converter",NULL,TRUE,NULL),NULL);
	default_mode_changed(gconf_client,-1,gconf_client_get_entry(gconf_client,"/apps/LinABX/general/default_mode",NULL,TRUE,NULL),NULL);
	restart_play_changed(gconf_client,-1,gconf_client_get_entry(gconf_client,"/apps/LinABX/samples/restart_play",NULL,TRUE,NULL),NULL);
	repeating_changed(gconf_client,-1,gconf_client_get_entry(gconf_client,"/apps/LinABX/samples/repeating",NULL,TRUE,NULL),NULL);
	statistics_changed(gconf_client,-1,gconf_client_get_entry(gconf_client,"/apps/LinABX/general/statistics_use_normal",NULL,TRUE,NULL),NULL);
	default_set_changed(gconf_client,-1,gconf_client_get_entry(gconf_client,"/apps/LinABX/samples/default_set",NULL,TRUE,NULL),NULL);
	only_reference_changed(gconf_client,-1,gconf_client_get_entry(gconf_client,"/apps/LinABX/samples/only_reference_sample",NULL,TRUE,NULL),NULL);
	a_sample_changed(gconf_client,-1,gconf_client_get_entry(gconf_client,"/apps/LinABX/samples/default_a_sample",NULL,TRUE,NULL),NULL);
	b_sample_changed(gconf_client,-1,gconf_client_get_entry(gconf_client,"/apps/LinABX/samples/default_b_sample",NULL,TRUE,NULL),NULL);
	audibility_threshold_changed(gconf_client,-1,gconf_client_get_entry(gconf_client,"/apps/LinABX/general/audibility_threshold",NULL,TRUE,NULL),NULL);
	
	gtk_widget_show(preferences);
    }
}


void
on_preferences_dialog_response         (GtkDialog       *dialog,
                                        gint             response_id,
                                        gpointer         user_data)
{
    if(response_id!=GTK_RESPONSE_HELP) {
        gtk_widget_destroy(preferences);
	gconf_client_notify_remove(gconf_client,default_folder_id);
	gconf_client_notify_remove(gconf_client,default_mode_id);
	gconf_client_notify_remove(gconf_client,default_folder_setting_id);
	gconf_client_notify_remove(gconf_client,restart_play_id);
	gconf_client_notify_remove(gconf_client,statistics_id);
	gconf_client_notify_remove(gconf_client,default_set_id);
	gconf_client_notify_remove(gconf_client,only_reference_id);
	gconf_client_notify_remove(gconf_client,a_sample_id);
	gconf_client_notify_remove(gconf_client,b_sample_id);
	gconf_client_notify_remove(gconf_client,audibility_threshold_id);
	preferences=NULL;
    }
}

void
on_cumulative_radiobutton_toggled      (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
    gconf_client_set_bool(gconf_client,"/apps/LinABX/general/statistics_use_normal",!gtk_toggle_button_get_active(togglebutton),NULL);
}


void
on_folder_not_change_radio_button_toggled
                                        (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
    if(gtk_toggle_button_get_active(togglebutton))
	gconf_client_set_int(gconf_client,"/apps/LinABX/general/default_folder_setting",0,NULL);
}


void
on_folder_goto_last_radiobutton_toggled
                                        (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
    if(gtk_toggle_button_get_active(togglebutton))
	gconf_client_set_int(gconf_client,"/apps/LinABX/general/default_folder_setting",1,NULL);
}



void
on_folder_set_to_radiobutton_toggled   (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
    if(gtk_toggle_button_get_active(togglebutton)) {
	gconf_client_set_int(gconf_client,"/apps/LinABX/general/default_folder_setting",2,NULL);
	gtk_widget_set_sensitive(lookup_widget(GTK_WIDGET(togglebutton),"folder_set_to_filechooser"),TRUE);
//	gtk_widget_grab_focus(lookup_widget(GTK_WIDGET(togglebutton),"folder_set_to_filechooser"));
//	gtk_widget_grab_default(lookup_widget(GTK_WIDGET(togglebutton),"folder_set_to_filechooser"));
    } else {
	gtk_widget_set_sensitive(lookup_widget(GTK_WIDGET(togglebutton),"folder_set_to_filechooser"),FALSE);
    }
}

void
on_folder_set_to_filechooser_file_activated
                                        (GtkFileChooser  *filechooser,
                                        gpointer         user_data)
{
    char *dir=gtk_file_chooser_get_filename(filechooser);
    gconf_client_set_string(gconf_client,"/apps/LinABX/general/default_folder",dir,NULL);
    g_free(dir);
}

void
on_default_no_mode_radiobutton_toggled    (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
    gconf_client_set_int(gconf_client,"/apps/LinABX/general/default_mode",0,NULL);
}

void
on_default_sample_radiobutton_toggled  (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
    gconf_client_set_int(gconf_client,"/apps/LinABX/general/default_mode",1,NULL);
}


void
on_default_jack_radiobutton_toggled    (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
    gconf_client_set_int(gconf_client,"/apps/LinABX/general/default_mode",2,NULL);
}


void
on_default_tone_radiobutton_toggled    (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
    gconf_client_set_int(gconf_client,"/apps/LinABX/general/default_mode",3,NULL);
}


void
on_restart_sample_checkbutton_toggled  (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
    gconf_client_set_bool(gconf_client,"/apps/LinABX/samples/restart_play",gtk_toggle_button_get_active(togglebutton),NULL);
}

void
on_repeating_checkbutton_toggled  (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
    gconf_client_set_bool(gconf_client,"/apps/LinABX/samples/repeating",gtk_toggle_button_get_active(togglebutton),NULL);
}

void
on_default_set_none_radiobutton_toggled
                                        (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
    gconf_client_set_int(gconf_client,"/apps/LinABX/samples/default_set",0,NULL);
}


void
on_default_set_fixed_radiobutton_toggled
                                        (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
    gconf_client_set_int(gconf_client,"/apps/LinABX/samples/default_set",1,NULL);
}


void
on_only_reference_checkbutton_toggled  (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
    gconf_client_set_bool(gconf_client,"/apps/LinABX/samples/only_reference_sample",gtk_toggle_button_get_active(togglebutton),NULL);
}


void
on_default_set_a_sample_filechooserbutton_current_folder_changed
                                        (GtkFileChooser  *filechooser,
                                        gpointer         user_data)
{
    char *file=gtk_file_chooser_get_filename(filechooser);
    if(file)
        gconf_client_set_string(gconf_client,"/apps/LinABX/samples/default_a_sample",file,NULL);
    g_free(file);
}


void
on_default_set_b_sample_filechooserbutton_current_folder_changed
                                        (GtkFileChooser  *filechooser,
                                        gpointer         user_data)
{
    char *file=gtk_file_chooser_get_filename(filechooser);
    if(file)
	gconf_client_set_string(gconf_client,"/apps/LinABX/samples/default_b_sample",file,NULL);
    g_free(file);
}

void
on_audibility_threshold_spinbutton_value_changed
                                        (GtkSpinButton   *spinbutton,
                                        gpointer         user_data)
{
    gconf_client_set_float(gconf_client,"/apps/LinABX/general/audibility_threshold",(float) gtk_spin_button_get_value(spinbutton),NULL);
}

