/* 
 * Copyright (C) 2005 Remco Poelstra <remco@beryllium.net>
 *
 * LinABX - main.c
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Authors: Remco Poelstra <remco@beryllium.net>
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>
#include <gconf/gconf-client.h>

#include "interface.h"
#include "support.h"

#include "jack.h"
#include "sample_mode.h"

#include "callbacks.h"

gboolean sample_mode=FALSE;
gboolean jack_mode=FALSE;
gboolean tone_mode=FALSE;
gboolean version=FALSE;
extern char *A_filename;
extern char *B_filename;
int converter=-1;
extern char **ports_from_options;
GConfClient *gconf_client;

GOptionEntry entries[] = {
  {"version",0,0,G_OPTION_ARG_NONE,&version,"Print the version number of LinABX and exit",NULL},
  {"sample-mode",'s',0,G_OPTION_ARG_NONE,&sample_mode,"Set LinABX in sample-mode",NULL},
  {"jack-mode",'j',0,G_OPTION_ARG_NONE,&jack_mode,"Set LinABX in jack-mode",NULL},
//  {"tone-mode",'t',0,G_OPTION_ARG_NONE,&tone_mode,"Set LinABX in tone-mode",NULL},
  {"converter-type",'c',0,G_OPTION_ARG_INT,&converter,"Set the samplerate converter to use to quality 'N'.\n\t\t\t     This is only usefull if LinABX is set to\n\t\t\t     sample-mode, otherwise it's ignored. Range from\n\t\t\t     0 (best) to 4 (worst)","N"},
  {NULL}
};

int
main (int argc, char *argv[])
{
  GtkWidget *mainwindow, *choose_dialog;
  GError *error=NULL;
  GOptionContext *context;
  
/*Parse commandline arguments*/
  context=g_option_context_new("- Determine audibility and audio artifacts or\n\
  train your ears in an audiotechnical way.\n\
  You can specify filenames or portnames as the last arguments. They will be\n\
  interpreted depending on the mode you selected. If your filename or portname\n\
  starts with a hyphen (-), then put a dash (--) in front, like:\n\
  linabx -s -- -file1.wav file2.wav");        
  g_option_context_add_main_entries(context,entries,NULL);
  g_option_context_add_group(context,gtk_get_option_group(TRUE));
  g_option_context_parse(context,&argc,&argv,&error);
  g_option_context_free(context);

  if(version) {
    printf("LinABX %s\n",VERSION);
    exit(0);
  }
  if(error) {
    printf("Error parsing commandline options. Unknown option or wrong value\n");
    exit(1);
  }

/* Init gnome, so we can go graphical*/
  gnome_program_init ("linabx", VERSION, LIBGNOMEUI_MODULE,
                      argc, argv,
		      GNOME_PARAM_APP_DATADIR,PACKAGE_DATA_DIR,
		      GNOME_PARAM_NONE);

/*Init the preferences*/
  gconf_client=gconf_client_get_default();
  gconf_client_add_dir(gconf_client,"/apps/LinABX",GCONF_CLIENT_PRELOAD_NONE,NULL);
  gconf_client_notify_add(gconf_client,"/apps/LinABX/samples/sample_rate_converter",converter_type_changed,NULL,NULL,NULL);
  converter_type_changed(gconf_client,-1,gconf_client_get_entry(gconf_client,"/apps/LinABX/general/sample_rate_converter",NULL,TRUE,NULL),NULL);  

/* Init jack so we can produce sound */
  init_jack();

/* Ok, we have sound. Create a GUI*/
  mainwindow = create_mainwindow ();

/*Show the main window*/
  gtk_widget_show (mainwindow);

/* Ask the user the first mode*/
  if(converter!=-1) {
     if(!((0<=converter)&&(converter<=4)))
	printf("Warning during option parsing: converter_type out of range. Using value stored in preferences\n");
    else
        gconf_client_set_int(gconf_client,"/apps/LinABX/samples/sample_rate_converter",converter,NULL);  
  }
  
  if(sample_mode+jack_mode+tone_mode!=1) {
    GConfEntry *entry=gconf_client_get_entry(gconf_client,"/apps/LinABX/general/default_mode",NULL,TRUE,NULL);
    if(sample_mode+jack_mode+tone_mode>1) printf("Warning during option parsing: Multiple modes specified\n");
    if(entry&&entry->value&&entry->value->type==GCONF_VALUE_INT&&gconf_value_get_int(entry->value)!=0) {
	int mode=gconf_value_get_int(entry->value);
	switch(mode) {
	    case 0:
		
		break;
	    case 1:
		gtk_menu_item_activate(GTK_MENU_ITEM(lookup_widget(mainwindow,"sample")));
		break;
	    case 2:
		gtk_menu_item_activate(GTK_MENU_ITEM(lookup_widget(mainwindow,"jack")));
		break;
	    case 3:
		gtk_menu_item_activate(GTK_MENU_ITEM(lookup_widget(mainwindow,"tone")));
		break;
	    default:
		goto choose;
		break;
	}
    } else {
	choose:
	choose_dialog=create_choose_mode_dialog();
	gtk_widget_show(choose_dialog);
	g_object_set_data(G_OBJECT(choose_dialog),"mainwindow",mainwindow);
    }
  } else {
    if(sample_mode) {
	if(argc>=2) {
	    int i=1;
	    if(strcmp(argv[1],"--")==0) i++;
	    if(i<argc) {
		if(!g_path_is_absolute(argv[i])) {
		    char *cwd=g_get_current_dir();
		    A_filename=g_build_filename(cwd,argv[i],NULL);
		    g_free(cwd);
		} else
		    A_filename=g_strdup(argv[i]);
	    }
	    i++;
	    if(i<argc) {
		if(!g_path_is_absolute(argv[i])) {
		    char *cwd=g_get_current_dir();
		    B_filename=g_build_filename(cwd,argv[i],NULL);
		    g_free(cwd);
		} else
		    B_filename=g_strdup(argv[i]);
	    }
	}
	gtk_menu_item_activate(GTK_MENU_ITEM(lookup_widget(mainwindow,"sample")));
    }
    else if(jack_mode) {
	if(argc>1) {
	int i=1;
	    ports_from_options=g_malloc(sizeof(char*)*4);
	    if(strcmp(argv[i],"--")==0) i++;
	    if(i<argc) ports_from_options[0]=g_strdup(argv[i]);
	    i++;
	    if(i<argc) ports_from_options[1]=g_strdup(argv[i]);
	    i++;
	    if(i<argc) ports_from_options[2]=g_strdup(argv[i]);
	    i++;
	    if(i<argc) ports_from_options[3]=g_strdup(argv[i]);
	}
	gtk_menu_item_activate(GTK_MENU_ITEM(lookup_widget(mainwindow,"jack")));
    }
    else if(tone_mode) {
	gtk_menu_item_activate(GTK_MENU_ITEM(lookup_widget(mainwindow,"tone")));
    }
  }

/*Accept user input*/
  gtk_main ();
  
/*We are done, exit*/
  close_jack();

  return 0;
}
