/* 
 * Copyright (C) 2005 Remco Poelstra <remco@beryllium.net>
 *
 * LinABX - sample_mode.c
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Authors: Remco Poelstra <remco@beryllium.net>
 */

#include "jack.h"
#include <jack/jack.h>
#include <jack/ringbuffer.h>
#include <jack/types.h>
#include "sample_mode.h"
#include <pthread.h>
#include <glib.h>
#include <sndfile.h>
#include <samplerate.h>
#include "callbacks.h"
#include <sys/types.h>

pthread_mutex_t disk_thread_lock=PTHREAD_MUTEX_INITIALIZER;
pthread_cond_t more_data = PTHREAD_COND_INITIALIZER;

extern jack_port_t *outputL;
extern jack_port_t *outputR;

extern enum mode_t mode;
extern int playing;

#define DEFAULT_RB_SIZE 16384

jack_ringbuffer_t *rbA_l;
jack_ringbuffer_t *rbA_r;
jack_ringbuffer_t *rbB_l;
jack_ringbuffer_t *rbB_r;

SNDFILE *A_file=NULL;
SNDFILE *B_file=NULL;
SRC_STATE *state=NULL;

char finish_thread=0;
pthread_t thread_id;

jack_nframes_t jack_rate=44100;
int file_rate=44100;
double ratio=1;
int mono=0;
int restart=1;
int repeating=1;

long get_new_data(void *cb_data, float **data) {
    static float out[4*1024*sizeof(float)];
    static float buffer[2*1024*sizeof(float)];
    int i;
    sf_count_t readA,readB;
    
    *data=out;
    if(!A_file||!B_file) {
	memset(*data,0,4*1024*sizeof(float));
	return 1024;
    }
    readA=sf_readf_float(A_file,buffer,1024);
    for(i=0;i<readA;i++) {
	if(!mono) {
	    (*data)[4*i]=buffer[2*i];
	    (*data)[4*i+1]=buffer[2*i+1];
	} else {
	    (*data)[4*i]=buffer[i];
	    (*data)[4*i+1]=buffer[+1];
	}
    }
    readB=sf_readf_float(B_file,buffer,1024);
    for(i=0;i<readB;i++) {
	if(!mono) {
	    (*data)[4*i+2]=buffer[2*i];
	    (*data)[4*i+3]=buffer[2*i+1];
	} else {
	    (*data)[4*i+2]=buffer[i];
	    (*data)[4*i+3]=buffer[i+1];
	}
    }
    /*Do we have to wrap around the file?*/
    if(readA<1024||readB<1024) {
	if(repeating) { /*do we actually want to wrap around the file?*/
    	    readA=readA<readB?readA:readB; /*get the smallest and store in readA for convinience*/
	    sf_seek(A_file,0,SEEK_SET);
	    sf_seek(B_file,0,SEEK_SET);
	    i=sf_readf_float(A_file,buffer,1024-readA);
    	    for(;i+readA>readA;i--) {
	        if(!mono) {
    	    	    (*data)[4*i+readA]=buffer[2*i];
		    (*data)[4*i+1+readA]=buffer[2*i+1];
		} else {
    		    (*data)[4*i+readA]=buffer[i];
		    (*data)[4*i+1+readA]=buffer[i+1];
		}
	    }
	    i=sf_readf_float(B_file,buffer,1024-readA);
	    for(;i+readA>readA;i--) {
	        if(!mono) {
	    	    (*data)[4*i+2+readA]=buffer[2*i];
		    (*data)[4*i+3+readA]=buffer[2*i+1];
		} else {
		    (*data)[4*i+2+readA]=buffer[i];
		    (*data)[4*i+3+readA]=buffer[i+1];
		}
	    }
	} else { /*no, we don't want to repeat, but let's reset the filepointer*/
	    set_playing(NO_ANSWER);
	    sf_seek(A_file,0,SEEK_SET);
	    sf_seek(B_file,0,SEEK_SET);
	}
    }
    return 1024;
}

void restart_play() {
    size_t free;
    
    if(mode!=SAMPLE_MODE) return;
    
    /*reset the file pointers*/
    sf_seek(A_file,0,SEEK_SET);
    sf_seek(B_file,0,SEEK_SET);
    /*clear the buffers*/
    free=jack_ringbuffer_read_space(rbA_l);
    jack_ringbuffer_read_advance(rbA_l,free);
    free=jack_ringbuffer_read_space(rbA_r);
    jack_ringbuffer_read_advance(rbA_r,free);
    free=jack_ringbuffer_read_space(rbB_l);
    jack_ringbuffer_read_advance(rbB_l,free);
    free=jack_ringbuffer_read_space(rbB_r);
    jack_ringbuffer_read_advance(rbB_r,free);
}
void get_audio_data(float *buffer) {
    if(ratio==1) {
	float *buffer_temp;
	get_new_data(NULL,&buffer_temp);
	memcpy(buffer,buffer_temp,1024*4*sizeof(float));
    } else src_callback_read(state,ratio,1024,buffer);

}

void *disk_thread(void *arg) {
    float *buffer=g_malloc(4*1024*sizeof(float));
    float *buffer_al=g_malloc(1024*sizeof(float));
    float *buffer_ar=g_malloc(1024*sizeof(float));
    float *buffer_bl=g_malloc(1024*sizeof(float));
    float *buffer_br=g_malloc(1024*sizeof(float));
    int i;
    
    pthread_setcanceltype(PTHREAD_CANCEL_ASYNCHRONOUS,NULL);
    pthread_mutex_lock(&disk_thread_lock);
    
    while(1) {
	while(jack_ringbuffer_write_space(rbA_l)>=1024*sizeof(float)&&!finish_thread) {
	    get_audio_data(buffer);
	    for(i=0;i<1024;i++) {
		buffer_al[i]=buffer[4*i];
		buffer_ar[i]=buffer[4*i+1];
		buffer_bl[i]=buffer[4*i+2];
		buffer_br[i]=buffer[4*i+3];
	    }
	    jack_ringbuffer_write(rbA_l,(void *)buffer_al,1024*sizeof(float));
	    jack_ringbuffer_write(rbA_r,(void *)buffer_ar,1024*sizeof(float));
	    jack_ringbuffer_write(rbB_l,(void *)buffer_bl,1024*sizeof(float));
	    jack_ringbuffer_write(rbB_r,(void *)buffer_br,1024*sizeof(float));
	}
	if(finish_thread) break;
	pthread_cond_wait(&more_data,&disk_thread_lock);
    }
    pthread_mutex_unlock(&disk_thread_lock);
    return 0;
}

int init_sample_mode(char *A_sample_file, char *B_sample_file, int converter_type) {
    SF_INFO info;
    int A_rate, channels;
    
    if(!A_sample_file||!B_sample_file)
	return 0;

    rbA_l=jack_ringbuffer_create(DEFAULT_RB_SIZE*sizeof(jack_default_audio_sample_t));
    rbA_r=jack_ringbuffer_create(DEFAULT_RB_SIZE*sizeof(jack_default_audio_sample_t));
    rbB_l=jack_ringbuffer_create(DEFAULT_RB_SIZE*sizeof(jack_default_audio_sample_t));
    rbB_r=jack_ringbuffer_create(DEFAULT_RB_SIZE*sizeof(jack_default_audio_sample_t));

    if(!rbA_l||!rbA_r||!rbB_l||!rbB_r) return 0;

    if(!(A_file=sf_open(A_sample_file,SFM_READ,&info)))
	return 0;

    A_rate=info.samplerate;
    channels=info.channels;
    
    if(!(B_file=sf_open(B_sample_file,SFM_READ,&info)))
	return 0;

    if(A_rate!=info.samplerate||channels!=info.channels) return 0;
    
    file_rate=A_rate;
    mono=channels>1?0:1;
    
    ratio=(double)jack_rate/file_rate;
    if(ratio!=1) {
    	if(!(state=src_callback_new(get_new_data,converter_type,4,NULL,NULL)))
	    return 0;
    }
    
    finish_thread=0;
    if(pthread_create(&thread_id,NULL,disk_thread,NULL)) return 0;
    mode=SAMPLE_MODE;
    if(ratio!=1) return 2;
    else return 1;
}

int process_samples(jack_nframes_t nframes, void *arg) {
    jack_default_audio_sample_t *outL= (jack_default_audio_sample_t *) jack_port_get_buffer(outputL,nframes);
    jack_default_audio_sample_t *outR= (jack_default_audio_sample_t *) jack_port_get_buffer(outputR,nframes);
    size_t bytes_per_port=nframes*sizeof(jack_default_audio_sample_t);
    
    switch(playing) {
	case A:
	    if(jack_ringbuffer_read_space(rbA_l)>=bytes_per_port) {
		jack_ringbuffer_read(rbA_l,(char *)outL,bytes_per_port);
		jack_ringbuffer_read(rbA_r,(char *)outR,bytes_per_port);
		jack_ringbuffer_read_advance(rbB_l,bytes_per_port);
		jack_ringbuffer_read_advance(rbB_r,bytes_per_port);
	    }
	    else {
	        memset(outL,0,sizeof(jack_default_audio_sample_t)*nframes);
		memset(outR,0,sizeof(jack_default_audio_sample_t)*nframes);
	    }
	    break;
	case B:
	    if(jack_ringbuffer_read_space(rbB_l)>=bytes_per_port) {
		jack_ringbuffer_read(rbB_l,(char *)outL,bytes_per_port);
		jack_ringbuffer_read(rbB_r,(char *)outR,bytes_per_port);
		jack_ringbuffer_read_advance(rbA_l,bytes_per_port);
		jack_ringbuffer_read_advance(rbA_r,bytes_per_port);
	    }
	    else {
	        memset(outL,0,sizeof(jack_default_audio_sample_t)*nframes);
		memset(outR,0,sizeof(jack_default_audio_sample_t)*nframes);
	    }
	    break;
	default:
	    memset(outL,0,sizeof(jack_default_audio_sample_t)*nframes);
	    memset(outR,0,sizeof(jack_default_audio_sample_t)*nframes);
    }
    
    if(pthread_mutex_trylock(&disk_thread_lock)==0) {
	pthread_cond_signal(&more_data);
	pthread_mutex_unlock(&disk_thread_lock);
    }
    return 0;
}

void cleanup_sample_mode(void) {
    finish_thread=1; /*Tell the disk_thread to stop*/
    pthread_join(thread_id,NULL);
    mode=NO_MODE;
    
    jack_ringbuffer_free(rbA_l);
    jack_ringbuffer_free(rbA_r);
    jack_ringbuffer_free(rbB_l);
    jack_ringbuffer_free(rbB_r);
    
    sf_close(A_file);
    A_file=NULL;
    sf_close(B_file);
    B_file=NULL;
    
    src_delete(state);
    state=NULL;
}

void update_sample_rate(jack_nframes_t srate) {
    jack_rate=srate;
}
